use strict;

package XML::SAXDriver::NYTimes;
use base qw (XML::SAX::Base);

=head1 NAME

XML::SAXDriver::NYTimes - generate DocBook SAX2 events for one or more New York Times articles.

=head1 SYNOPSIS

 use IO::AtomicFile;
 use XML::SAX::Writer;
 use XML::SAXDriver::NYTimes;

 my $output  = IO::AtomicFile->open("/tmp/times.dbk","w");
 my $writer  = XML::SAX::Writer->new(Output=>$output);
 my $handler = XML::SAXDriver::NYTimes->new(Handler=>$writer);

 $handler->set_cookies_file("mozilla","/path/to/cookies");
 $handler->fetch();

 $output->close();

=head1 DESCRIPTION

Generate DocBook SAX2 events for one or more New York Times articles.

There is no deep magic happening here. Just some good old fashioned
scraping which means it may break at a later date.

=cut

use Date::Calc qw(Day_of_Week check_date);

use HTML::Parser;
use HTTP::Request;

use LWP::UserAgent;
use URI;

$XML::SAXDriver::NYTimes::VERSION = '0.6';

use constant NYTIMES                => "http://www.nytimes.com";
use constant MAGAZINE               => "/library/magazine/home";
use constant PRINTED                => "pagewanted=print";
use constant INDEX                  => "/info/help/textpath.html";

use constant DEFAULT_SECTION_MATCH  => "^(\/pages\/)([a-z\/]+)(\/text)(.*)";
use constant DEFAULT_HEADLINE_MATCH => "^\/((\d\d\d\d)\/(\d\d)\/(\d\d))(.*)";

#

my $ua      = undef;

#

my %articles  = ();
my @sections  = ();
my @headlines = ();

my $sections_match  = undef;
my $headlines_match = undef;

my $login = 0;

=head1 PACKAGE METHODS

=head2 $pkg = __PACKAGE__->new()

This package inherits from I<XML::SAX::Base>. Please consult the docs for details.

Returns an object. Woot!

=cut

sub new {
  my $pkg = shift;

  $ua = LWP::UserAgent->new();

  if (! $ua) {
    carp $!;
    return undef;
  }

  $ua->agent("Mozilla/5.0");

  return $pkg->SUPER::new(@_);
}

=head1 OBJECT METHODS

=head2 $pkg->set_cookies()

This function has not been implemented yet.

=cut

=head2 $pkg->set_cookies_file($browser,$path_to_cookies)

Load a file containing your New York Times cookies.

I<$browser> must be a valid sub-class of I<HTTP::Cookies>

=cut

sub set_cookies_file {
  my $self    = shift;
  my $browser = shift;
  my $cookies = shift;

  my $class = "HTTP::Cookies::".ucfirst(lc($browser));
  eval "require $class";

  if ($@) {
    $XML::SAXDriver::NYTimes::Logger::logger->critical("Failed to load cookie jar for $browser, $@\n");
    return 0;
  }

  my $jar = $class->new(File=>$cookies,AutoSave => 1);

  if (! $jar) {
    $XML::SAXDriver::NYTimes::Logger::logger->critical("Failed to create cookie jar for $cookies, $!\n");
    return 0;
  }

  $ua->cookie_jar($jar);
  return 1;
}

=head2 $pkg->set_log_dispatcher($obj)

Set one or more I<Log::Dispatch> thingies for reporting
errors and general verbosity.

=cut

sub set_log_dispatcher {
  my $self = shift;
  $XML::SAXDriver::NYTimes::Logger::logger->add(@_);
}

=head2 $pkg->fetch($date)

String.

Dates should be passed as "yyyy/mm/dd". If no date is specified then 
the date returned by I<localtime> is assumed.

=cut

sub fetch {
  my $self = shift;
  my $date = shift;

  $date ||=  XML::SAXDriver::NYTimes::Date->today();

  $self->start_document();
  $self->start_book();

  foreach my $path (@{&_sections()}) {

    $path =~ /^(\/pages\/)([a-z\/]+)(\/text)(.*)/;
    my $section = ucfirst $2;

    my $url   = NYTIMES.$path;

    my $match = $date;
    $match    =~ s!/!\\/!g;
    $match    = "^\\/($match)(.*)";

    $self->_section($url,$match);
  }

  #

  if (&Date::Calc::Day_of_Week(split("/",$date)) eq "7") {
    $self->_magazine($date);
  }

  $self->end_book();
  $self->end_document();

  return 1;
}

=head2 $pkg->fetch_section($section,$date)

String, string.

Generate a DocBook I<book> document, containing (1) part and (n)
articles for I<$section>

Dates should be passed as "yyyy/mm/dd". If no date is specified then
the date returned by I<localtime> is assumed.

=cut

sub fetch_section {
  my $self    = shift;
  my $section = shift;
  my $date    = shift || XML::SAXDriver::NYTimes::Date->today();

  my $url   = undef;
  my $match = undef;

  foreach my $path (@{&_sections}) {
    $path =~ /^(\/pages\/)($section)(\/text)(.*)$/;

    if ($2) {
      $url   = NYTIMES.$path;
      $match = $date;

      $match    =~ s!/!\\/!g;
      $match    = "^\\/($match)(.*)";
      last;
    }
  }

  if ((! $url) || (! $match)) {
    $XML::SAXDriver::NYTimes::Logger::logger->notice("Unable to find section named '$section'\n");
    return 0;
  }

  $self->start_document();
  $self->start_book();
  $self->_section($url,$match);
  $self->end_book();
  $self->end_document();

  return 1;
}

=head2 $pkg->fetch_article($url)

String.

Generate a DocBook I<article> document for I<url>.

=cut

sub fetch_article {
  my $self = shift;
  $self->start_document();
  $self->_article(@_);
  $self->end_document();
  return 1;
}

=head2 $pkg->sections()

Returns an array ref of sections listed by the New York Times.

=cut

sub sections {
  my $self = shift;
  my $sections = &_sections();

  foreach my $s (@$sections) {
    $s =~ /^(\/pages\/)([a-z\/]+)(\/text)(.*)/;
    $s = $2;
  }

  return $sections;
}

#

sub _section {
  my $self  = shift;
  my $url   = shift;
  my $match = shift;

  my $links = &_headlines($url,$match);

  if (! scalar(@$links)) {
    $XML::SAXDriver::NYTimes::Logger::logger->notice("Unable to find any links for $url.\n");
    return 0;
  }

  $url =~ /^(.*)\/pages\/(.*)\/text(.*)$/;
  my $title = $2;

  $self->start_part($title);
  $self->_articles($links);
  $self->end_part();

  return 1;
}

sub _article {
  my $self = shift;
  my $url  = shift;

  if (! $url) {
    $XML::SAXDriver::NYTimes::Logger::logger->error("No url to parse.\n");
    return 0;
  }

  # Check to make sure it is a N.Y. Times URL

  my $uri  = URI->new($url) || return (undef,$!);
  my $host = $uri->host()   || return (undef,"Unable to determine host.");

  unless ($host =~ /(nytimes\.com)$/) {
      warn "AHHHH '$host'\n";
    return (undef,"Unable to verify nytimes-ness of the URL.");
  }

  # Rather than checking to see if we have 
  # the correct query string for a "printable"
  # version, we just blast the query string
  # and tack on the correct CGI parameters.

  my $print_url = (split(/\?/,$url))[0];
  $print_url = join("\?",$print_url,PRINTED);

  my $res = $ua->request(HTTP::Request->new(GET => $print_url));

  if (! $res->is_success()) {
    $XML::SAXDriver::NYTimes::Logger::logger->warning("Can't get $print_url : ".$res->code());
    return 0;
  }

  #

  if ((! $login) && (${$res->content_ref()} =~ /<title>Register at NYTimes\.com/)) {
    $XML::SAXDriver::NYTimes::Logger::logger->error("Ack login failed!\n");
    return 0;
  }

  $login = 1;

  #

  $self->SUPER::start_element({Name=>"article",Attributes=>{
							    "{}id" => {
								       Name => "id",
								       LocalName => "id",
								       Prefix => "",
								       NamespaceURI => "",
								       Value => $url,
								      }
							   }});


  my $parser = XML::SAXDriver::NYTimes::Parser->new();
  $parser->xml_mode(1);

  $parser->parse($res->content_ref(),$self);
  $self->SUPER::end_element({Name=>"article"});

  return 1;
}

sub _xref {
  my $self = shift;
  my $url  = shift;

  # print "XREF-ing '$url'\n";
  $self->SUPER::start_element({Name=>"xref",Attributes=>{
							 "{}linkend" => {
									 Name => "linkend",
									 LocalName => "linkend",
									 NamespaceURI => "",
									 Prefix       => "",
									 Value => $url,
									}
							}});
  $self->SUPER::end_element({Name=>"xref"});
}

sub _magazine {
  my $self = shift;
  my $date = shift;

  my $root = NYTIMES;
  my $url  = $root.MAGAZINE;
  
  $date    =~ s!/!\\/!g;
  $date    = "\\/($date)\\/magazine";
  
  my $links = &_headlines($url,$date);

  if (! scalar(@$links)) {
    $XML::SAXDriver::NYTimes::Logger::logger->notice("It's Sunday, but I was unable to locate any links for the Magazine.");
    return 0;
  }

  # I can't remember why we need to jump
  # through these hoops, but we do.
  map { $_ =~ s/^$root//; } @$links;

  $self->start_part("Magazine");
  $self->_articles($links);
  $self->end_part();

  return 1;
}

sub _articles {
  my $self  = shift;
  my $links = shift;

  foreach my $link (@$links) {

    if ($articles{NYTIMES.$link}) {
      $XML::SAXDriver::NYTimes::Logger::logger->debug("REFERENCING '".NYTIMES.$link."'\n");
      $self->_xref(NYTIMES.$link);
      next;
    }

    $self->_article(NYTIMES.$link);
    $XML::SAXDriver::NYTimes::Logger::logger->debug("RECORDING '".NYTIMES.$link."'\n");
    $articles{NYTIMES.$link} ++;
  }

}

#

sub start_document {
  my $self = shift;
  $self->SUPER::start_document();
  $self->SUPER::xml_decl({Version=>"1.0",Encoding=>"ISO-8859-1"});
}

sub end_document {
  my $self = shift;
  $self->SUPER::end_document();
}

sub start_book {
  my $self = shift;

  $self->SUPER::start_element({Name=>"book"});
  $self->SUPER::start_element({Name=>"bookinfo"});
  $self->SUPER::start_element({Name=>"title"});
  $self->SUPER::characters({Data=>"New York Times for ".XML::SAXDriver::NYTimes::Date->today()});
  $self->SUPER::end_element({Name=>"title"});
  $self->SUPER::start_element({Name=>"pubdate"});
  $self->SUPER::characters({Data=>XML::SAXDriver::NYTimes::Date->pubdate()});
  $self->SUPER::end_element({Name=>"pubdate"});
  $self->SUPER::end_element({Name=>"bookinfo"});

  return 1;
}

sub end_book {
  my $self = shift;
  $self->SUPER::end_element({Name=>"book"});
}

sub start_part {
  my $self  = shift;
  my $title = shift;

  $title = join(": ", map { $_ = ucfirst(lc($_)); } split("/",$title));

  $self->SUPER::start_element({Name=>"part"});
  $self->SUPER::start_element({Name=>"partinfo"});
  $self->SUPER::start_element({Name=>"title"});
  $self->SUPER::characters({Data=>$title});
  $self->SUPER::end_element({Name=>"title"});
  $self->SUPER::end_element({Name=>"partinfo"});
}

sub end_part {
  my $self = shift;
  $self->SUPER::end_element({Name=>"part"});
}

#

sub _sections {
  my $pattern = shift;

  &_sections_match($pattern);
  @sections = ();
  
  my $p = HTML::Parser->new(start_h=>[\&_parse_sections,"tagname,attr"]);
  my $res = $ua->request(HTTP::Request->new(GET=>NYTIMES.INDEX), sub {$p->parse($_[0])});
  
  return \@sections;
}

sub _parse_sections {
  my($tag, $attr) = @_;
  return if $tag ne 'a';
  my $url = $attr->{'href'};
  return unless ($url =~ /$sections_match/);
  push(@sections, $url);
}

sub _sections_match {
    my $pattern = shift;
    $sections_match = ($pattern) ? $pattern : DEFAULT_SECTION_MATCH; 
}

#

sub _headlines {
  my $url     = shift;
  my $pattern = shift;
  
  &_headlines_match($pattern);
  @headlines  = (); 

  my $p   = HTML::Parser->new(start_h=>[\&_parse_headlines,"tagname,attr"]);
  my $res = $ua->request(HTTP::Request->new(GET => $url), sub { $p->parse($_[0])});
  
  return \@headlines;
}

sub _parse_headlines {
  my($tag, $attr) = @_;
  return if $tag ne 'a';
  my $url = $attr->{'href'};
  return unless ($url =~ /$headlines_match/);
  push(@headlines, $url) unless (grep /^($url)$/,@headlines);
}

sub _headlines_match {
  my $pattern = shift;
  $headlines_match = ($pattern) ? $pattern : DEFAULT_HEADLINE_MATCH; 
  $XML::SAXDriver::NYTimes::Logger::logger->debug("Matching '$headlines_match'\n");
}

package XML::SAXDriver::NYTimes::Date;
use Date::Format;

use constant PUBDATE => "%Y-%m-%dT%H:%M:%S %z";

sub pubdate {
  my $pkg  = shift;
  my $time = shift || time;
  return (ref($time) eq "ARRAY") ? strftime(PUBDATE,@$time) : time2str(PUBDATE,$time);
}

sub today {
    my @date = (localtime)[5,4,3];
    $date[0] = $date[0] + 1900;
    $date[1] = $date[1] + 1;
    if ($date[1] < 10) { $date[1] = "0".$date[1]; }
    if ($date[2] < 10) { $date[2] = "0".$date[2]; }
    return join("/",@date);
}

package XML::SAXDriver::NYTimes::Parser;
use base qw (HTML::Parser);

use String::Random qw(random_string);

use constant META_ABSTRACT  => "lp";
use constant META_ARTICLEID => "articleid";
use constant META_BYLINE    => "byl";
use constant META_GEO       => "geo";
use constant META_HEADLINE  => "hdl";
use constant META_KEYWORDS  => "des";
use constant META_ORG       => "org";
use constant META_PERSON    => "per";
use constant META_PUBDATE   => "pdate";

my %ranges = ();
my $okay   = 0;

sub init {
  my $self = shift;
  $self->SUPER::init(start_h => [\&_start,"self,tagname,attr"],
		     end_h   => [\&_end,"self,tagname"],
		     text_h  => [\&_text,"self,text"]);
}

sub parse {
  my $self    = shift;
  my $content = shift;
  my $handler = shift;

  %ranges = ();

  $self->{Handler} = $handler;
  $self->SUPER::parse($$content);
}

sub _mk_rangeid {
    my $str = random_string("cccccccc");

    if (exists($ranges{$str})) {
	return &_mk_rangeid();
    }

    $ranges{ $str } ++;
    return $str;
}

sub _start {
  my $self = shift;
  my $tag   = shift;
  my $attrs = shift;

  if ($tag =~ /^(nyt_text)/i) { 
    $okay = 1;
  }

  # if ($tag eq "nyt_links_onsite") { $okay = 0; }

  if (($okay) && ($tag =~ /^img$/i)) {
    my $src = $attrs->{src} || $attrs->{SRC};

    if ($src =~ /\/dropcap\/\w\.gif$/) {
      $self->{dropcap} = ($attrs->{'alt'} || $attrs->{'ALT'});
    }
  }

  if ($tag eq "head") {
    $self->{Handler}->start_element({Name=>"articleinfo"});
  }
  
  if (($tag eq "meta") && ($attrs->{'name'} eq META_HEADLINE)) { 
    my $title = $attrs->{'content'} || "";

    $self->{Handler}->start_element({Name=>"title"});
    $self->{Handler}->characters({Data=>$title});
    $self->{Handler}->end_element({Name=>"title"});
  }
  
  if (($tag eq "meta") && ($attrs->{'name'} eq META_BYLINE)) { 

    $attrs->{'content'} =~ /^By(?:\s)*((?:[^\s]+)\s?.*)$/i;
    my @authors = split(" and ",$1);

    if (@authors) {
	$self->{Handler}->start_element({Name=>"authorgroup"});
	map { $self->_author($_) } @authors;
	$self->{Handler}->end_element({Name=>"authorgroup"});
    }

  }

  if (($tag eq "meta") && ($attrs->{'name'} eq META_ABSTRACT)) { 
    $self->{Handler}->start_element({Name=>"abstract"});

    my @paras = 
	grep {
	    $_ =~ /\w/;
	} map {
	    $_ =~ s/\s+//;
	    $_ =~ s/^\.$//;
	    $_;
	} (split(";",$attrs->{content}));

    foreach my $p (@paras) {
	$self->{Handler}->start_element({Name=>"para"});
	$self->{Handler}->characters({Data=>$p});	
	$self->{Handler}->end_element({Name=>"para"});
    }

    $self->{Handler}->end_element({Name=>"abstract"});
  }
  
  if (($tag eq "meta") && ($attrs->{'name'} eq META_PUBDATE)) { 
    $attrs->{content} =~ /^(\d{4})(\d{2})(\d{2})/;
    my @d = (undef,undef,undef,$3,($2 -1),($1 -1900));
    $self->{Handler}->start_element({Name=>"pubdate"});
    $self->{Handler}->characters({Data=>XML::SAXDriver::NYTimes::Date->pubdate(\@d)});
    $self->{Handler}->end_element({Name=>"pubdate"});
  }
  
  if (($tag eq "meta") && ($attrs->{'name'} eq META_KEYWORDS)) { 

      if (my @kw = split(";",$attrs->{content})) {

	  $self->{Handler}->start_element({Name=>"keywordset"});

	  foreach my $w (@kw) {
	      $self->{Handler}->start_element({Name=>"keyword"});
	      $self->{Handler}->characters({Data=>$w});
	      $self->{Handler}->end_element({Name=>"keyword"});
	  }

	  $self->{Handler}->end_element({Name=>"keywordset"});
      }
  }

  if (($tag eq "meta") && ($attrs->{'name'} eq META_PERSON)) { 
      $self->_indexterm($attrs);
  }

  if (($tag eq "meta") && ($attrs->{'name'} eq META_GEO)) { 
      $self->_indexterm($attrs);
  }

  if (($tag eq "meta") && ($attrs->{'name'} eq META_ORG)) { 
      $self->_indexterm($attrs);
  }

  if (($tag eq "meta") && ($attrs->{'name'} eq META_ARTICLEID)) { 
      $self->_indexterm($attrs);
  }

}

sub _author {
    my $self = shift;
    my $name = shift;

    if ($name =~ /^the/i) {
	$self->_corpauthor($name);
    }

    elsif ($name =~ /\.com$/) {
	$self->_corpauthor($name);
    }

    elsif ($name =~ /news$/) {
	$self->_corpauthor($name);
    }

    elsif ($name =~ /^reuters$/) {
	$self->_corpauthor($name);
    }

    else {
	my $firstname = undef;
	my $surname   = undef;
	my $othername = undef;

	$name =~ /^(\w+)\s*(.*)$/;
	
	$firstname = ucfirst(lc($1));
	$surname   = $2;
	
	if ($surname =~ /^(?:(\w\.)?\s)?(\w+)$/) {
	    $othername = $1;
	    $surname   = $2;
	}

	$surname = ucfirst(lc($surname));

	$self->{Handler}->start_element({Name=>"author"});
	$self->{Handler}->start_element({Name=>"surname"});
	$self->{Handler}->characters({Data=>$surname});
	$self->{Handler}->end_element({Name=>"surname"});
	$self->{Handler}->start_element({Name=>"firstname"});
	$self->{Handler}->characters({Data=>$firstname});
	$self->{Handler}->end_element({Name=>"firstname"});

	if ($othername) {
	    $self->{Handler}->start_element({Name=>"othername"});
	    $self->{Handler}->characters({Data=>$othername});
	    $self->{Handler}->end_element({Name=>"othername"});
	}

	$self->{Handler}->end_element({Name=>"author"});
    }
}

sub _corpauthor {
    my $self = shift;
    my $name = shift;

    my $corpname = undef;

    if ($name =~ /\.com$/) {
	$corpname = $name;
    } else {
	$corpname = join(" ",map {
	    $_ = ucfirst(lc($_));
	    $_;
	} split(" ",$name));
    }

    $self->{Handler}->start_element({Name=>"corpauthor"});
    $self->{Handler}->characters({Data=>$corpname});
    $self->{Handler}->end_element({Name=>"corpauthor"});
}

sub _indexterm {
    my $self  = shift;
    my $attrs = shift;

    if (! $attrs->{content}) {
	return;
    }

    #

    my @terms = split(";",$attrs->{content});

    if (scalar(@terms) == 1) {
	$self->{Handler}->start_element({Name       => "indexterm",
					 Attributes => { "{}type" => { Name         => "type",
								       LocalName    => "type",
								       Prefix       => "",
								       NamespaceURI => "",
								       Value        => $attrs->{'name'} },
						     }});
	
	$self->{Handler}->start_element({Name=>"primary"});
	$self->{Handler}->characters({Data=>$terms[0]});
	$self->{Handler}->end_element({Name=>"primary"});
	$self->{Handler}->end_element({Name=>"indexterm"});      

	return;
    }

    # 

    my $indexid = &_mk_rangeid();
    my $first   = shift @terms;

    $self->{Handler}->start_element({Name       => "indexterm",
				     Attributes => {"{}id" => { Name           => "id",
								  LocalName    => "id",
								  Prefix       => "",
								  NamespaceURI => "",
								   Value       => $indexid },
						    "{}class" => { Name         => "class",
								  LocalName    => "class",
								  Prefix       => "",
								  NamespaceURI => "",
								  Value        => "startofrange" },
						    "{}type" => { Name         => "type",
								  LocalName    => "type",
								  Prefix       => "",
								  NamespaceURI => "",
								   Value        => $attrs->{'name'} },
						 }});
    
    $self->{Handler}->start_element({Name=>"primary"});
    $self->{Handler}->characters({Data=>$first});
    $self->{Handler}->end_element({Name=>"primary"});
    $self->{Handler}->end_element({Name=>"indexterm"});      

    foreach my $t (@terms) {
	$self->{Handler}->start_element({Name       => "indexterm",
					 Attributes => { "{}type" => { Name         => "type",
								       LocalName    => "type",
								       Prefix       => "",
								       NamespaceURI => "",
								       Value        => $attrs->{'name'} },
						     }});
	
	$self->{Handler}->start_element({Name=>"primary"});
	$self->{Handler}->characters({Data=>$t});
	$self->{Handler}->end_element({Name=>"primary"});
	$self->{Handler}->end_element({Name=>"indexterm"});      
    }

    $self->{Handler}->start_element({Name       => "indexterm",
				     Attributes => {"{}id" => { Name           => "startref",
								  LocalName    => "startef",
								Prefix       => "",
								NamespaceURI => "",
								Value       => $indexid },
						    "{}class" => { Name         => "class",
								   LocalName    => "class",
								   Prefix       => "",
								   NamespaceURI => "",
								   Value        => "endofrange" },
						    "{}type" => { Name         => "type",
								  LocalName    => "type",
								  Prefix       => "",
								  NamespaceURI => "",
								  Value        => $attrs->{'name'} },
						}});
    $self->{Handler}->end_element({Name=>"indexterm"});

    return;
}

sub _end { 
  my $self = shift;
  my $tag = shift;

  if ($tag  =~ /^(nyt_text)$/i) { 
    $okay = 0;
  }

  if ($tag eq "head") {

    $self->{Handler}->start_element({Name=>"copyright"});
    $self->{Handler}->start_element({Name=>"year"});
    $self->{Handler}->characters({Data=>(split("/",XML::SAXDriver::NYTimes::Date->today()))[0]});
    $self->{Handler}->end_element({Name=>"year"});
    $self->{Handler}->start_element({Name=>"holder"});
    $self->{Handler}->characters({Data=>"New York Times Corporation"});
    $self->{Handler}->end_element({Name=>"holder"});
    $self->{Handler}->end_element({Name=>"copyright"});

    $self->{Handler}->end_element({Name=>"articleinfo"});
  }

}

sub _text {
  my $self = shift;
  my $text = shift;

  return unless ($text);

  $text =~ s/^\s+//;
  $text =~ s/\s$//;

  return unless ($text && $okay);

  $self->{Handler}->start_element({Name=>"para"});

  if ($self->{dropcap}) {
    $self->{Handler}->characters({Data=>$self->{dropcap}});
    delete $self->{dropcap};
  }

  $self->{Handler}->characters({Data=>$text});
  $self->{Handler}->end_element({Name=>"para"});
}

package XML::SAXDriver::NYTimes::Logger;
use vars qw ($logger);

use Log::Dispatch;
$logger = Log::Dispatch->new();

=head1 TERMS OF USAGE

This software is meant for personal use only, in accordance with the New York Times terms of usage :

 "You may download or copy the Content and other downloadable items displayed on the 
  Service for personal use only, provided that you maintain all copyright and other 
  notices contained therein. Copying or storing of any Content for other than personal 
  use is expressly prohibited without prior written permission from The New York Times 
  Rights and Permissions Department, or the copyright holder identified in the copyright 
  notice contained in the Content."

  http://www.nytimes.com/ref/membercenter/help/agree.html#sect2

Play nicely.

=head1 DOCUMENT STRUCTURE

This package generates DocBook SAX2 events with either a I<book>
or an I<article> root element.

Book widgets contain the following elements :

 + book
   + bookinfo
     - title
     - pubdate
   + part (+)
     + partinfo
       - title
     + xref (+)
       @linkend
     + article (+)

Article widgets contain the following elements :

 + article
   @id
   + articleinfo
     - title
     + keywordset (or not)
       - keyword
     + indexterm (or not)
       @ type
       - primary
     + authorgroup
       - corpauthor
       OR
       + author
        - surname
        - firstname
        - othername (or not)
     + abstract
       - para
     - pubdate
     + copyright
       - year
       - holder
   - para (+)

=cut

=head1 VERSION

0.6

=head1 DATE

$Date: 2004/03/05 22:50:52 $

=head1 AUTHOR

Aaron Straup Cope

=head1 BACKGROUND

One day a friend wrote :

 "You know, what I would really like in a bot, though I'm sure there are
  reasons this doesn't exist. is a bot that you can send a NYTimes URL
  to and it will spit back [or email you? does this break some rule?] the
  text of the article, no banners, no headlines, no cloying logins etc.
  I have been dealing with a particularly zealous fan who has been trying
  to tell me that linking to NYT articles is Bad Form since not everyone
  has a login, even though I do offer my own login to them ... So, I'm
  thinking I should learn perl in order to be able to do this."

And that was all it took.

=head1 SEE ALSO

http://aaronland.info/weblog/archive/2868

http://aaronland.info/weblog/archive/3266

http://aaronland.info/weblog/archive/4124

=head1 TO DO

=over 4

=item *

Finish I<set_cookies()> so you can pass a hash ref rather than the path to a file

=back

=head1 BUGS

=over 4

=item *

The code for determining where to add <para> tags is crude, at best, and may result in weirdness.

=back

=head1 LICENSE

Copyright (c) 2002-2004, Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under the same terms as Perl itself

=cut

return 1;
